@@ -4,6 +4,7 @@
 
 #pragma once
 #include <frc/GenericHID.h>
+#include <wpi/deprecated.h>
 
 #include "Button.h"
 
@@ -20,11 +21,13 @@ class JoystickButton : public Button {
    * Creates a JoystickButton that commands can be bound to.
    *
    * @param joystick The joystick on which the button is located.
-   * @param buttonNumber The number of the button on the joystic.
+   * @param buttonNumber The number of the button on the joystick.
    */
+  WPI_IGNORE_DEPRECATED
   explicit JoystickButton(std::shared_ptr<frc::GenericHID> joystick, int buttonNumber)
       : Button([joystick, buttonNumber] {
           return joystick->GetRawButton(buttonNumber);
         }) {}
+  WPI_UNIGNORE_DEPRECATED
 };
 }  // namespace frc2
